package cn.estsh.i3plus.ext.mes.pcn.apiservice.controller.busi;

import cn.estsh.i3plus.ext.mes.pcn.api.job.IMesOffLineDataSyncJobService;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.config.MesOffLineConfig;
import cn.estsh.i3plus.ext.mes.pcn.pojo.constant.MesCommonConstant;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.pojo.mes.bean.MesPcnSyncOfflineLog;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.SpringContextsUtil;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.net.InetAddress;
import java.util.HashMap;
import java.util.Map;

@RestController
@RequestMapping(MesCommonConstant.MES_YANFEN + "/offline-data-sync")
public class MesOffLineDataSyncController {

    @Autowired
    private IMesOffLineDataSyncJobService offLineDataSyncJobService;

    @GetMapping("/cfg/get")
    @ApiOperation(value = "获取离线配置")
    public ResultBean getOffLineCfg() {
        try {
            MesOffLineConfig offLineConfig = (MesOffLineConfig) SpringContextsUtil.getBean("mesOffLineConfig");
            Map<String, Object> resultMap = new HashMap<>();
            resultMap.put("dataSource", offLineConfig.getDataSource());
            resultMap.put("pcnTask", offLineConfig.getPcnTask());
            resultMap.put("sqlParameter", offLineConfig.getSqlParameter());
            resultMap.put("sql", offLineConfig.getSql());
            return ResultBean.success("查询成功").setResultMap(resultMap);
        } catch (ImppBusiException imppException) {
            return ResultBean.fail(imppException);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping("/do")
    @ApiOperation(value = "执行离线同步")
    public ResultBean doOffLineDataSync() {
        try {

            MesOffLineConfig offLineConfig = (MesOffLineConfig) SpringContextsUtil.getBean("mesOffLineConfig");

            String param = offLineConfig.getPcnTask().getTaskParam();

            String[] params = StringUtils.isEmpty(param) ? null : param.split(MesPcnExtConstWords.COLON);

            if (null == params || params.length != 4) return ResultBean.fail("参数配置错误,参数正确格式[ mes.offline.sync.pcnTask.taskParam = 工厂:区域:生产线:工位 ]");

            MesPcnSyncOfflineLog pcnSyncOfflineLog = new MesPcnSyncOfflineLog(params[0], params[1], params[2], params[3]);
            pcnSyncOfflineLog.setSyncTarget(InetAddress.getLocalHost().getHostAddress());

            return ResultBean.success(offLineDataSyncJobService.doOffLineDataSync(pcnSyncOfflineLog));
        } catch (ImppBusiException imppException) {
            return ResultBean.fail(imppException);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

}
