package cn.estsh.i3plus.ext.mes.pcn.apiservice.schedulejob;


import cn.estsh.i3plus.ext.mes.pcn.api.job.IMesOffLineDataSyncJobService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.pojo.mes.bean.MesPcnSyncOfflineLog;
import com.alibaba.fastjson.JSONObject;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StopWatch;

import java.net.InetAddress;

/**
 * @Author: Wynne.Lu
 * @CreateDate: 2019/12/17 1:18 下午
 * @Description:
 **/
@Slf4j
@ApiOperation("MES同步离线数据JOB")
@DisallowConcurrentExecution
public class MesOffLineDataSyncJob implements Job {

    @Autowired
    private IMesOffLineDataSyncJobService offLineDataSyncJobService;

    public MesOffLineDataSyncJob() {}

    @Override
    public void execute(JobExecutionContext jobExecutionContext) {

        log.info("MES同步离线数据JOB --- START ---");

        StopWatch stopWatch = new StopWatch();

        stopWatch.start();

        try {

            Object param = jobExecutionContext.getJobDetail().getJobDataMap().get("param");

            if (null == param) {
                log.info("MES同步离线数据JOB --- ERROR --- 未配置参数 ---");
                return;
            }

            String[] params = param.toString().split(MesPcnExtConstWords.COLON);

            if (null == params || params.length != 4) {
                log.info("MES同步离线数据JOB --- ERROR --- 参数配置错误,参数正确格式[ mes.offline.sync.pcnTask.taskParam = 工厂:区域:生产线:工位 ] ---");
                return;
            }

            MesPcnSyncOfflineLog pcnSyncOfflineLog = new MesPcnSyncOfflineLog(params[0], params[1], params[2], params[3]);
            pcnSyncOfflineLog.setSyncTarget(InetAddress.getLocalHost().getHostAddress());

            String success = offLineDataSyncJobService.doOffLineDataSync(pcnSyncOfflineLog);

            log.info("MES同步离线数据JOB --- EXEC --- {} ---", success);

        } catch (Exception e) {

            log.info("MES同步离线数据JOB --- ERROR --- {} ---", JSONObject.toJSONString(e));

        } finally {

            stopWatch.stop();

            log.info("MES同步离线数据JOB --- END --- 耗时:{} ms ---", stopWatch.getTotalTimeMillis());

        }

    }
}
