package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.job;

import cn.estsh.i3plus.ext.mes.pcn.api.job.IMesOffLineDataSyncJobService;
import cn.estsh.i3plus.ext.mes.pcn.api.job.IMesOffLineDataSyncService;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.config.MesOffLineConfig;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.mes.pcn.serviceimpl.base.PcnJobService;
import cn.estsh.i3plus.mes.pcn.util.BsJdbcTemplate;
import cn.estsh.i3plus.mes.pcn.util.BsJdbcTemplateConfig;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.mes.bean.MesDatasource;
import cn.estsh.i3plus.pojo.mes.bean.MesPcnSyncOfflineLog;
import cn.estsh.i3plus.pojo.mes.bean.MesPcnTask;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.util.SpringContextsUtil;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;

@Slf4j
@Primary
@Service
public class MesOffLineDataSyncJobService extends PcnJobService implements IMesOffLineDataSyncJobService {

    private MesOffLineConfig offLineConfig;

    private BsJdbcTemplateConfig jdbcTemplateConfig;

    @Autowired
    private IMesOffLineDataSyncService offLineDataSyncService;

    @Override
    public List<MesPcnTask> loadJobs(Boolean pcnScheduleJob, Boolean mesOffLineSyncOpen) {
        List<MesPcnTask> list = super.loadJobs(pcnScheduleJob, mesOffLineSyncOpen);
        if (!mesOffLineSyncOpen) return list;
        try {
            offLineConfig = (MesOffLineConfig) SpringContextsUtil.getBean("mesOffLineConfig");
            jdbcTemplateConfig = (BsJdbcTemplateConfig) SpringContextsUtil.getBean("bsJdbcTemplateConfig");
            list.add(offLineConfig.getPcnTask());
        } catch (Exception e) {
        }
        return list;
    }

    @Override
    public String doOffLineDataSync(MesPcnSyncOfflineLog pcnSyncOfflineLog) {

        if (null == offLineConfig) offLineConfig = (MesOffLineConfig) SpringContextsUtil.getBean("mesOffLineConfig");

        if (null == offLineConfig) return "MesOffLineConfig IS NULL";

        MesDatasource datasource = offLineConfig.getDataSource();
        pcnSyncOfflineLog.setSyncSource(String.format("mysql://%s:%s/%s", datasource.getDsHost(), datasource.getDsPort(), datasource.getDsDbName()));
        pcnSyncOfflineLog.setSyncPattern("全量同步");
        pcnSyncOfflineLog.setSyncDate(TimeTool.getToday());
        ConvertBean.serviceModelInitialize(pcnSyncOfflineLog, MesPcnExtConstWords.JOB);
        pcnSyncOfflineLog.setSystemSyncDatetime(pcnSyncOfflineLog.getModifyDatetime());

        String error = checkDataSourceIsValid(datasource);
        if (!StringUtils.isEmpty(error)) return error;

        Map<String, String> sqlMap = offLineConfig.getSql();
        Map<String, String> sqlParameterMap = offLineConfig.getSqlParameter();

        if (CollectionUtils.isEmpty(sqlMap) || CollectionUtils.isEmpty(sqlParameterMap)) return "SQL CFG IS NULL";

        List<MesPcnSyncOfflineLog> pcnSyncOfflineLogList = offLineDataSyncService.getPcnSyncOfflineLogList(pcnSyncOfflineLog.getOrganizeCode());

        List<String> successedList = offLineDataSyncService.filterPcnSyncOfflineLogList(pcnSyncOfflineLogList, true);

        if (!CollectionUtils.isEmpty(successedList) && successedList.containsAll(sqlMap.keySet())) return String.format("%s已同步成功,无需重复执行!", sqlMap.keySet().toString());

        Map<String, String> failuredMap = offLineDataSyncService.filterFaulurePcnSyncOfflineLog(pcnSyncOfflineLogList);

        if (null == jdbcTemplateConfig) jdbcTemplateConfig = (BsJdbcTemplateConfig) SpringContextsUtil.getBean("bsJdbcTemplateConfig");

        BsJdbcTemplate bsJdbcTemplate = null != jdbcTemplateConfig ? jdbcTemplateConfig.getJdbcTemplate(datasource) : null;

        Boolean isConn = null != jdbcTemplateConfig ? jdbcTemplateConfig.checkDbConn(datasource) : false;

        String tableName;
        Boolean success = true;
        List<String> successTableNameList = new ArrayList<>();
        List<String> failureTableNameList = new ArrayList<>();

        for (String objectCode : sqlMap.keySet()) {

            if (StringUtils.isEmpty(objectCode)) continue;

            if (!objectCode.contains(MesPcnExtConstWords.DECIMAL_POINT)) {
                tableName = objectCode;
            } else {
                //如果pojo在 DEFAULT_POJO_ROUTE 下面还有包, 配置的时候需带上除去 DEFAULT_POJO_ROUTE 以外的包
                String[] suffix = objectCode.split(MesPcnExtConstWords.DECIMAL_POINT_ESCAPE);
                tableName = suffix[suffix.length - 1];
            }

            if (!CollectionUtils.isEmpty(successedList) && successedList.contains(tableName)) continue;

            Boolean status = false;
            String remark = null;

            try {

                String sql = sqlMap.get(objectCode);
                if (StringUtils.isEmpty(sql)) {
                    remark = "SQL IS NULL";
                    continue;
                }
                if (null == jdbcTemplateConfig) {
                    remark = "BsJdbcTemplateConfig IS NULL";
                    continue;
                }
                if (null == bsJdbcTemplate) {
                    remark = "BsJdbcTemplate IS NULL";
                    continue;
                }
                if (!isConn) {
                    remark = "checkDbConn IS FALSE";
                    continue;
                }

                Map<String, Object> paramMap = new HashMap<>();

                for (Map.Entry<String, String> sqlParameter : sqlParameterMap.entrySet()) {

                    if (null == sqlParameter || StringUtils.isEmpty(sqlParameter.getValue())) continue;

                    if (!sql.contains(MesPcnExtConstWords.COLON + sqlParameter.getKey())) continue;

                    if (sqlParameter.getKey().contains("_int")) paramMap.put(sqlParameter.getKey(), Integer.valueOf(sqlParameter.getValue()));
                    else if (sqlParameter.getKey().contains("_list")) paramMap.put(sqlParameter.getKey(), Arrays.asList(sqlParameter.getValue().split(MesPcnExtConstWords.COMMA)));
                    // TODO else if
                    else paramMap.put(sqlParameter.getKey(), sqlParameter.getValue());

                }

                List<Map<String, Object>> resultList = bsJdbcTemplate.getJdbcTemplate().queryForList(sql, paramMap);

                if (CollectionUtils.isEmpty(resultList)) remark = "未查询到数据";
                else {
                    offLineDataSyncService.insertBaseBeanData(pcnSyncOfflineLog, resultList, objectCode, tableName);
                    successTableNameList.add(tableName);
                    status = true;
                }

            } catch (Exception e) {

                remark = e.toString();

                success = false;

                log.info("MES同步离线数据JOB --- ERROR --- {} ---", JSONObject.toJSONString(e));

            } finally {
                if (!status) {
                    insertPcnSyncOfflineLog2Failure(pcnSyncOfflineLog, tableName, failuredMap, remark);
                    failureTableNameList.add(String.format("[%s]同步失败原因[%s]", tableName, remark));
                    success = false;
                }
            }

        }

        if (!CollectionUtils.isEmpty(successTableNameList) && success) {

            StringBuilder builder = new StringBuilder();
            builder.append(" insert into mes_pcn_sync_offline_log ");
            builder.append("( id, organize_code, is_valid, is_deleted, create_user, create_date_time, modify_user, modify_date_time, system_sync_date_time, system_sync_status,");
            builder.append(" area_code, work_center_code, work_cell_code, object_code, effect_num, sync_date, sync_pattern, sync_source, sync_target, sync_result )");
            builder.append(" values ( :id , :organizeCode , :isValid , :isDeleted , :createUser , :createDatetime , :modifyUser , :modifyDatetime , :systemSyncDatetime , :systemSyncStatus ,");
            builder.append(" :areaCode , :workCenterCode , :workCellCode , :objectCode , :effectNum , :syncDate , :syncPattern , :syncSource , :syncTarget , :syncResult ); ");

            Map<String, Object> paramMap = JSONObject.parseObject(JSONObject.toJSONString(pcnSyncOfflineLog), Map.class);
            paramMap.put(MesPcnExtConstWords.OBJECT_CODE, sqlMap.keySet().toString());
            paramMap.put(MesPcnExtConstWords.EFFECT_NUM, MesPcnExtConstWords.ZERO);

            bsJdbcTemplate.getJdbcTemplate().update(builder.toString(), paramMap);

        }

        return (CollectionUtils.isEmpty(successTableNameList) ? MesPcnExtConstWords.EMPTY : String.format("%s同步成功!", successTableNameList.toString())) +
                (CollectionUtils.isEmpty(failureTableNameList) ? MesPcnExtConstWords.EMPTY : failureTableNameList.toString());

    }

    private String checkDataSourceIsValid(MesDatasource datasource) {

        if (null == datasource) return "MesDatasource IS NULL";

        if (StringUtils.isEmpty(datasource.getDsCode())) return "离线边端数据库[dsCode]未配置!";
        if (StringUtils.isEmpty(datasource.getDsName())) return "离线边端数据库[dsName]未配置!";
        if (StringUtils.isEmpty(datasource.getDsType())) return "离线边端数据库[dsType]未配置!";
        if (StringUtils.isEmpty(datasource.getDsHost())) return "离线边端数据库[dsHost]未配置!";
        if (StringUtils.isEmpty(datasource.getDsPort())) return "离线边端数据库[dsHostPort]未配置!";
        if (StringUtils.isEmpty(datasource.getDsUser())) return "离线边端数据库[dsUser]未配置!";
        if (StringUtils.isEmpty(datasource.getDsPassword())) return "离线边端数据库[dsPassword]未配置!";
        if (StringUtils.isEmpty(datasource.getDsDbName())) return "离线边端数据库[dsDbName]未配置!";

        return null;
    }

    private void insertPcnSyncOfflineLog2Failure(MesPcnSyncOfflineLog pcnSyncOfflineLog, String objectCode, Map<String, String> failuredMap, String remark) {

        if (!CollectionUtils.isEmpty(failuredMap) && failuredMap.containsKey(objectCode) && failuredMap.get(objectCode).equals(remark)) return;

        offLineDataSyncService.insertPcnSyncOfflineLog(pcnSyncOfflineLog.syncResult(MesExtEnumUtil.MES_LOG_DEAL_STATUS.DEAL_FAILURE.getValue()).remark(remark), objectCode);

    }

}
