package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.job;

import cn.estsh.i3plus.ext.mes.pcn.api.job.IMesOffLineDataSyncService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.mes.pcn.util.StringUtil;
import cn.estsh.i3plus.platform.common.tool.ReflectTool;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.BaseBean;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.jpa.dao.BaseRepository;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesPcnSyncOfflineLog;
import cn.estsh.i3plus.pojo.mes.repository.MesPcnSyncOfflineLogRepository;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.util.SpringContextsUtil;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ArrayUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.persistence.Column;
import java.lang.reflect.Field;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
public class MesOffLineDataSyncService implements IMesOffLineDataSyncService {

    @Autowired
    private MesPcnSyncOfflineLogRepository pcnSyncOfflineLogRepository;

    private static String DEFAULT_POJO_ROUTE = "cn.estsh.i3plus.pojo.mes.bean.";

    @Override
    public List<MesPcnSyncOfflineLog> getPcnSyncOfflineLogList(String organizeCode) {
        return pcnSyncOfflineLogRepository.findByProperty(
                new String[]{MesPcnExtConstWords.ORGANIZE_CODE, MesPcnExtConstWords.IS_DELETED, MesPcnExtConstWords.IS_VALID, MesPcnExtConstWords.SYNC_DATE},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), CommonEnumUtil.IS_VAILD.VAILD.getValue(), TimeTool.getToday()});
    }

    @Override
    public List<String> filterPcnSyncOfflineLogList(List<MesPcnSyncOfflineLog> pcnSyncOfflineLogList, Boolean flag) {
        return CollectionUtils.isEmpty(pcnSyncOfflineLogList) ? null : pcnSyncOfflineLogList.stream().filter(o -> (
                null != o && o.getSyncResult().compareTo(MesExtEnumUtil.MES_LOG_DEAL_STATUS.getValueByFlag(flag)) == 0)).map(MesPcnSyncOfflineLog::getObjectCode).collect(Collectors.toList());
    }

    @Override
    public Map<String, String> filterFaulurePcnSyncOfflineLog(List<MesPcnSyncOfflineLog> pcnSyncOfflineLogList) {
        List<MesPcnSyncOfflineLog> filterList = CollectionUtils.isEmpty(pcnSyncOfflineLogList) ? null :
                pcnSyncOfflineLogList.stream().filter(o -> (null != o && o.getSyncResult().compareTo(MesExtEnumUtil.MES_LOG_DEAL_STATUS.DEAL_FAILURE.getValue()) == 0)).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(filterList)) return null;
        filterList = filterList.stream().filter(o -> null != o).sorted(Comparator.comparing(MesPcnSyncOfflineLog::getCreateDatetime).reversed()).collect(Collectors.toList());
        filterList = filterList.stream().filter(o -> null != o).distinct().collect(Collectors.collectingAndThen(
                Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(MesPcnSyncOfflineLog::getObjectCode))), ArrayList::new));
        return filterList.stream().filter(o -> null != o).collect(Collectors.toMap(MesPcnSyncOfflineLog::getObjectCode, MesPcnSyncOfflineLog::getRemark));
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRES_NEW, noRollbackFor = {ImppBusiException.class, Exception.class})
    public MesPcnSyncOfflineLog insertPcnSyncOfflineLog(MesPcnSyncOfflineLog pcnSyncOfflineLog, String tableName) {
        pcnSyncOfflineLog.setObjectCode(tableName);
        pcnSyncOfflineLog.setId(null);
        pcnSyncOfflineLog = pcnSyncOfflineLogRepository.insert(pcnSyncOfflineLog);
        log.info("MES同步离线数据JOB --- EXEC --- TABLE_NAME: {} --- {} ---", tableName, JSONObject.toJSONString(pcnSyncOfflineLog));
        return pcnSyncOfflineLog;
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRES_NEW, noRollbackFor = {ImppBusiException.class, Exception.class})
    public void insertBaseBeanData(MesPcnSyncOfflineLog pcnSyncOfflineLog, List<Map<String, Object>> resultList, String objectCode, String tableName) throws Exception {

        String daoName = StringUtil.lineToHump(tableName);
        String pojoName = StringUtil.toUpperCaseFirst(daoName);
        String pojoRoute = DEFAULT_POJO_ROUTE + (!objectCode.contains(MesPcnExtConstWords.DECIMAL_POINT) ? pojoName : objectCode.replace(tableName, pojoName));

        BaseRepository baseRepository = (BaseRepository) SpringContextsUtil.getBean(daoName + MesPcnExtConstWords.Repository);

        List<BaseBean> historyList = baseRepository.findAll();
        Map<Long, BaseBean> historyMap = CollectionUtils.isEmpty(historyList) ? null : historyList.stream().filter(o -> null != o).collect(Collectors.toMap(BaseBean::getId, o -> o));

        Class clazz = Class.forName(pojoRoute);

        Field[] fields = ArrayUtils.addAll(clazz.getFields(), clazz.getDeclaredFields());

        String fieldName;
        for (Map<String, Object> map : resultList) {

            BaseBean baseBean = (BaseBean) clazz.newInstance();

            for (Field field : fields) {

                if (null == field) continue;

                Column column = field.getAnnotation(Column.class);

                if (null == column) continue;

                fieldName = column.name().toLowerCase();

                if (!map.containsKey(fieldName)) continue;

                ReflectTool.setFieldValue(baseBean, field, map.get(fieldName));

            }

            baseBean.setSystemSyncDatetime(pcnSyncOfflineLog.getSystemSyncDatetime());
            if (!CollectionUtils.isEmpty(historyMap) && historyMap.containsKey(baseBean.getId())) {
                baseRepository.updateNoSync(baseBean);
                historyMap.remove(baseBean.getId());
            } else {
                baseRepository.insert(baseBean);
            }

        }

        historyList = CollectionUtils.isEmpty(historyMap) ? null :
                historyMap.values().stream().filter(o -> (null != o && o.getIsDeleted().compareTo(CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue()) == 0)).collect(Collectors.toList());

        if (!CollectionUtils.isEmpty(historyList)) {
            DdlPackBean packBean = DdlPackBean.getDdlPackBean(pcnSyncOfflineLog.getOrganizeCode());
            DdlPreparedPack.getInPackList(historyList.stream().filter(o -> null != o).map(BaseBean::getId).collect(Collectors.toList()), MesPcnExtConstWords.ID, packBean);
            baseRepository.updateByProperties(MesPcnExtConstWords.IS_DELETED, CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue(), packBean);
        }

        insertPcnSyncOfflineLog(pcnSyncOfflineLog.syncResult(MesExtEnumUtil.MES_LOG_DEAL_STATUS.DEAL_SUCCESS.getValue()).effectNum(CollectionUtils.isEmpty(resultList) ? 0 : resultList.size()).remark(null), tableName);

    }

}
